//
//  BMKGeofenceMenuView.m
//  BMKGeofenceOCDemo
//
//  Created by v_chiyuanwei on 2020/7/15.
//  Copyright © 2020 baidu. All rights reserved.
//

#import "BMKGeofenceMenuView.h"


@interface BMKGeofenceMenuView () <UITextFieldDelegate>

@property (nonatomic, strong) UIButton *circleFenceBtn;
@property (nonatomic, strong) UIButton *polygonFenceBtn;
@property (nonatomic, strong) UITextField *fenceRadiusTF;
@property (nonatomic, strong) UITextField *fenceIDTF;
@property (nonatomic, strong) UIView *sptrline;
@property (nonatomic, strong) UIStackView *activeActionStackView;
@property (nonatomic, strong) UILabel *activeActionLabel;
@property (nonatomic, strong) UIButton *activeActionInBtn;
@property (nonatomic, strong) UIButton *activeActionOutBtn;
@property (nonatomic, strong) UIButton *activeActionStayBtn;
@property (nonatomic, strong) UILabel *fenceCenterLabel;

/// 围栏类型 1：圆 2：多边形
@property (nonatomic, assign) NSInteger fenceType;
/// 围栏半径
@property (nonatomic, assign) CGFloat fenceRadius;
/// 围栏ID
@property (nonatomic, copy) NSString *fenceID;
/// 触发条件
@property (nonatomic, assign) BMKGeoFenceActiveAction activeAction;

@end

@implementation BMKGeofenceMenuView

- (instancetype)initWithFrame:(CGRect)frame {
    
    if (self = [super initWithFrame:frame]) {
        [self initialize];
        [self setupViews];
    }
    
    return  self;
}

- (void)initialize {
    self.backgroundColor = [UIColor whiteColor];
    _activeAction = BMKGeoFenceActiveActionInside;
    _fenceType = 1;
}

- (void)setupViews {
    [self addSubview:self.circleFenceBtn];
    [self addSubview:self.polygonFenceBtn];
    [self addSubview:self.fenceRadiusTF];
    [self addSubview:self.fenceIDTF];
    [self addSubview:self.sptrline];
    [self addSubview:self.activeActionStackView];
    [self addSubview:self.fenceCenterLabel];
}

- (void)layoutSubviews {
    [super layoutSubviews];
    
    self.circleFenceBtn.frame = CGRectMake(15, 15, 110, 20);
    self.polygonFenceBtn.frame = CGRectMake(15, CGRectGetMaxY(self.circleFenceBtn.frame) + 10, 110, 20);
    self.fenceIDTF.frame = CGRectMake(self.frame.size.width - 80 - 15, 12.5, 80, 25);
    self.fenceRadiusTF.frame = CGRectMake(self.fenceIDTF.frame.origin.x, CGRectGetMaxY(self.fenceIDTF.frame) + 5, 80, 25);
    self.sptrline.frame = CGRectMake(15, CGRectGetMaxY(self.polygonFenceBtn.frame) + 10, self.frame.size.width - 30, 0.5);
    self.activeActionStackView.frame = CGRectMake(15, CGRectGetMaxY(self.sptrline.frame) + 10, 260, 25);
    self.fenceCenterLabel.frame = CGRectMake(15, CGRectGetMaxY(self.activeActionStackView.frame) + 10, self.frame.size.width - 30, 20);
    
}

#pragma mark - Action
- (void)FenceBtnAction:(UIButton *)btn {
    if (btn.selected) { return; }

    if (btn == self.circleFenceBtn) {
        self.circleFenceBtn.selected = YES;
        self.polygonFenceBtn.selected = NO;
        self.fenceType = 1;
        self.fenceRadiusTF.hidden = NO;
        self.coordsInfoStr = @"请点击地图选择地图中心点";
    } else {
        self.circleFenceBtn.selected = NO;
        self.polygonFenceBtn.selected = YES;
        self.fenceType = 2;
        self.fenceRadiusTF.hidden = YES;
        self.coordsInfoStr = @"请点击地图选择围栏的边界点，至少3个";
    }
    
    if (self.fenceTypeChangedBlock) {
        self.fenceTypeChangedBlock(self.fenceType);
    }
    
    [self.polygonFenceCoords removeAllObjects];
}

- (void)activeActionBtnAction:(UIButton *)btn {
    btn.selected = !btn.isSelected;
    
    if (self.activeActionInBtn.selected) {
        self.activeAction = self.activeAction | BMKGeoFenceActiveActionInside;
    }else {
        self.activeAction = self.activeAction & (~BMKGeoFenceActiveActionInside);
    }
    
    if (self.activeActionOutBtn.selected) {
        self.activeAction = self.activeAction | BMKGeoFenceActiveActionOutside;
    } else {
        self.activeAction = self.activeAction & (~BMKGeoFenceActiveActionOutside);
    }

    if (self.activeActionStayBtn.selected) {
        self.activeAction = self.activeAction | BMKGeoFenceActiveActionStayed;
    } else {
        self.activeAction = self.activeAction & (~BMKGeoFenceActiveActionStayed);
    }
}

- (void)textFieldValueChanged:(UITextField *)textField {
    if (textField == self.fenceRadiusTF) {
        self.fenceRadius = [textField.text floatValue];
    } else {
        self.fenceID = textField.text;
    }
}

#pragma mark - setter
- (void)setFenceCenterCoord:(CLLocationCoordinate2D)fenceCenterCoord {
    _fenceCenterCoord = fenceCenterCoord;
    
    self.coordsInfoStr = [NSString stringWithFormat:@"围栏中心：lat:%f,lon:%f", fenceCenterCoord.latitude, fenceCenterCoord.longitude];
}

- (void)setCoordsInfoStr:(NSString *)coordsInfoStr {
    _coordsInfoStr = coordsInfoStr;
    
    self.fenceCenterLabel.text = coordsInfoStr;
    
    if ([coordsInfoStr containsString:@"请点击"]) {
        self.fenceCenterLabel.backgroundColor = [UIColor redColor];
    } else {
        self.fenceCenterLabel.backgroundColor = [UIColor whiteColor];
    }
}

#pragma mark - getter
- (UIButton *)circleFenceBtn {
    if (!_circleFenceBtn) {
        _circleFenceBtn = [UIButton buttonWithType:UIButtonTypeCustom];
        [_circleFenceBtn setImage:[UIImage imageNamed:@"package_rb_unchecked"] forState:UIControlStateNormal];
        [_circleFenceBtn setImage:[UIImage imageNamed:@"package_rb_checked"] forState:UIControlStateSelected];
        _circleFenceBtn.titleLabel.font = [UIFont systemFontOfSize:14];
        [_circleFenceBtn setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
        _circleFenceBtn.contentHorizontalAlignment = UIControlContentHorizontalAlignmentLeft;
        [_circleFenceBtn setTitle:@"  圆形围栏" forState:UIControlStateNormal];
        _circleFenceBtn.selected = YES;
        
        [_circleFenceBtn addTarget:self action:@selector(FenceBtnAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _circleFenceBtn;
}

- (UIButton *)polygonFenceBtn {
    if (!_polygonFenceBtn) {
        _polygonFenceBtn = [UIButton buttonWithType:UIButtonTypeCustom];
        [_polygonFenceBtn setImage:[UIImage imageNamed:@"package_rb_unchecked"] forState:UIControlStateNormal];
        [_polygonFenceBtn setImage:[UIImage imageNamed:@"package_rb_checked"] forState:UIControlStateSelected];
        _polygonFenceBtn.titleLabel.font = [UIFont systemFontOfSize:14];
        [_polygonFenceBtn setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
        _polygonFenceBtn.contentHorizontalAlignment = UIControlContentHorizontalAlignmentLeft;
        [_polygonFenceBtn setTitle:@"  多边形围栏" forState:UIControlStateNormal];

        [_polygonFenceBtn addTarget:self action:@selector(FenceBtnAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _polygonFenceBtn;
}

- (UITextField *)fenceRadiusTF {
    if (!_fenceRadiusTF) {
        _fenceRadiusTF = [[UITextField alloc] init];
        _fenceRadiusTF.font = [UIFont systemFontOfSize:14];
        _fenceRadiusTF.keyboardType = UIKeyboardTypeDecimalPad;
        _fenceRadiusTF.placeholder = @"围栏半径";
        _fenceRadiusTF.layer.borderWidth = 0.5;
        _fenceRadiusTF.layer.borderColor = [UIColor blackColor].CGColor;
        _fenceRadiusTF.layer.cornerRadius = 4;
        _fenceRadiusTF.layer.masksToBounds = YES;
        
        [_fenceRadiusTF addTarget:self action:@selector(textFieldValueChanged:) forControlEvents:UIControlEventEditingChanged];
    }
    return _fenceRadiusTF;
}

- (UITextField *)fenceIDTF {
    if (!_fenceIDTF) {
        _fenceIDTF = [[UITextField alloc] init];
        _fenceIDTF.font = [UIFont systemFontOfSize:14];
        _fenceIDTF.placeholder = @"业务ID";
        _fenceIDTF.layer.borderWidth = 0.5;
        _fenceIDTF.layer.borderColor = [UIColor blackColor].CGColor;
        _fenceIDTF.layer.cornerRadius = 4;
        _fenceIDTF.layer.masksToBounds = YES;
        
        [_fenceIDTF addTarget:self action:@selector(textFieldValueChanged:) forControlEvents:UIControlEventEditingChanged];
    }
    return _fenceIDTF;
}

- (UIView *)sptrline {
    if (!_sptrline) {
        _sptrline = [[UIView alloc] init];
        _sptrline.backgroundColor = [[UIColor blackColor] colorWithAlphaComponent:0.5];
    }
    return _sptrline;
}

- (UIStackView *)activeActionStackView {
    if (!_activeActionStackView) {
        NSArray *views = @[self.activeActionLabel, self.activeActionInBtn, self.activeActionOutBtn, self.activeActionStayBtn];
        _activeActionStackView = [[UIStackView alloc] initWithArrangedSubviews:views];
        _activeActionStackView.axis = UILayoutConstraintAxisHorizontal;
        _activeActionStackView.spacing = 10;
    }
    return _activeActionStackView;
}

- (UILabel *)activeActionLabel {
    if (!_activeActionLabel) {
        _activeActionLabel = [[UILabel alloc] init];
        _activeActionLabel.font = [UIFont systemFontOfSize:14];
        _activeActionLabel.text = @"触发条件";
    }
    return _activeActionLabel;
}

- (UIButton *)activeActionInBtn {
    if (!_activeActionInBtn) {
        _activeActionInBtn = [UIButton buttonWithType:UIButtonTypeCustom];
        [_activeActionInBtn setImage:[UIImage imageNamed:@"checkbox_normal"] forState:UIControlStateNormal];
        [_activeActionInBtn setImage:[UIImage imageNamed:@"checkbox_pressed"] forState:UIControlStateSelected];
        _activeActionInBtn.titleLabel.font = [UIFont systemFontOfSize:14];
        [_activeActionInBtn setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
        [_activeActionInBtn setTitle:@"  进入" forState:UIControlStateNormal];
        _activeActionInBtn.selected = YES;
        
        [_activeActionInBtn addTarget:self action:@selector(activeActionBtnAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _activeActionInBtn;
}

- (UIButton *)activeActionOutBtn {
    if (!_activeActionOutBtn) {
        _activeActionOutBtn = [UIButton buttonWithType:UIButtonTypeCustom];
        [_activeActionOutBtn setImage:[UIImage imageNamed:@"checkbox_normal"] forState:UIControlStateNormal];
        [_activeActionOutBtn setImage:[UIImage imageNamed:@"checkbox_pressed"] forState:UIControlStateSelected];
        _activeActionOutBtn.titleLabel.font = [UIFont systemFontOfSize:14];
        [_activeActionOutBtn setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
        [_activeActionOutBtn setTitle:@"  离开" forState:UIControlStateNormal];
        [_activeActionOutBtn addTarget:self action:@selector(activeActionBtnAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _activeActionOutBtn;
}

- (UIButton *)activeActionStayBtn {
    if (!_activeActionStayBtn) {
        _activeActionStayBtn = [UIButton buttonWithType:UIButtonTypeCustom];
        [_activeActionStayBtn setImage:[UIImage imageNamed:@"checkbox_normal"] forState:UIControlStateNormal];
        [_activeActionStayBtn setImage:[UIImage imageNamed:@"checkbox_pressed"] forState:UIControlStateSelected];
        _activeActionStayBtn.titleLabel.font = [UIFont systemFontOfSize:14];
        [_activeActionStayBtn setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
        [_activeActionStayBtn setTitle:@"  停留" forState:UIControlStateNormal];
        [_activeActionStayBtn addTarget:self action:@selector(activeActionBtnAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _activeActionStayBtn;
}

- (UILabel *)fenceCenterLabel {
    if (!_fenceCenterLabel) {
        _fenceCenterLabel = [[UILabel alloc] init];
        _fenceCenterLabel.font = [UIFont systemFontOfSize:14];
        _fenceCenterLabel.backgroundColor = [UIColor redColor];
        _fenceCenterLabel.text = @"请点击地图选择地图中心点";
    }
    return _fenceCenterLabel;
}

- (NSMutableArray<NSValue *> *)polygonFenceCoords {
    if (!_polygonFenceCoords) {
        _polygonFenceCoords = [NSMutableArray array];
    }
    return _polygonFenceCoords;
}

@end
